@@ -1,31 +1,106 @@
 `ensemble.test.splits` <- function(
-    x, p, a=NULL, an=1000, ext=NULL, k=5, digits=2, PLOTS=FALSE, SCRIPT=TRUE,
-    MAXENT=1, GBM=1, GBMSTEP=1, RF=1, GLM=1, GLMSTEP=1, GAM=1, GAMSTEP=1, 
-    MGCV=1, EARTH=1, RPART=1, NNET=1, FDA=1, SVM=1, BIOCLIM=1, DOMAIN=1, MAHAL=1,   
-    Yweights="BIOMOD", factors=NULL, formulae.defaults=TRUE,
-    GBM.formula=NULL, GBM.n.trees=3000,
-    GBMSTEP.gbm.x=2:(1+nlayers(x)), GBMSTEP.tree.complexity=5, GBMSTEP.learning.rate=0.01, GBMSTEP.bag.fraction=0.5,
+    x, p, a=NULL, an=1000, ext=NULL, k=5, 
+    layer.drops=NULL, VIF=FALSE,
+    digits=2, PLOTS=FALSE, SCRIPT=TRUE,
+    input.weights=NULL,
+    MAXENT=1, GBM=1, GBMSTEP=1, RF=1, GLM=1, GLMSTEP=1, GAM=1, GAMSTEP=1, MGCV=1, MGCVFIX=0, 
+    EARTH=1, RPART=1, NNET=1, FDA=1, SVM=1, SVME=1, BIOCLIM=1, DOMAIN=1, MAHAL=1, 
+    GEODIST=0, GEODIST.file.name="Species001", RASTER.format="raster",
+    Yweights="BIOMOD", factors=NULL, dummy.vars=NULL,
+    formulae.defaults=TRUE, maxit=100,
+    GBM.formula=NULL, GBM.n.trees=2000,
+    GBMSTEP.gbm.x=2:(1+nlayers(x)), GBMSTEP.tree.complexity=5, GBMSTEP.learning.rate=0.005, 
+    GBMSTEP.bag.fraction=0.5, GBMSTEP.step.size=100,
     RF.formula=NULL, RF.ntree=750, RF.mtry=log(nlayers(x)), 
     GLM.formula=NULL, GLM.family=binomial(link="logit"), 
     GLMSTEP.steps=1000, STEP.formula=NULL, GLMSTEP.scope=NULL, GLMSTEP.k=2,
     GAM.formula=NULL, GAM.family=binomial(link="logit"), 
     GAMSTEP.steps=1000, GAMSTEP.scope=NULL,
-    MGCV.formula=NULL,
-    EARTH.formula=NULL, EARTH.glm=list(family=binomial(link="logit"), maxit=1000),
+    MGCV.formula=NULL, MGCV.select=FALSE,
+    MGCVFIX.formula=NULL, 
+    EARTH.formula=NULL, EARTH.glm=list(family=binomial(link="logit"), maxit=maxit),
     RPART.formula=NULL, RPART.xval=50, 
     NNET.formula=NULL, NNET.size=8, NNET.decay=0.01,
     FDA.formula=NULL,
-    SVM.formula=NULL  
+    SVM.formula=NULL, SVME.formula=NULL  
 )
 {
     .BiodiversityR <- new.env()
+    if (k < 1) {stop("Parameter k smaller than 1")}
     if (! require(dismo)) {stop("Please install the dismo package")}
+    if (is.null(layer.drops) == F) {
+        vars <- names(x)
+        layer.drops <- as.character(layer.drops)
+        factors <- as.character(factors)
+        dummy.vars <- as.character(dummy.vars)
+        nd <- length(layer.drops)
+        for (i in 1:nd) {     
+            if (any(vars==layer.drops[i])==FALSE) {
+                cat(paste("\n", "WARNING: variable to exclude '", layer.drops[i], "' not among grid layers", "\n", sep = ""))
+            }else{
+                cat(paste("\n", "NOTE: variable ", layer.drops[i], " will not be included as explanatory variable", "\n", sep = ""))
+                x <- dropLayer(x, which(names(x) == layer.drops[i]))
+                vars <- names(x)
+                if (is.null(factors) == F) {
+                    factors <- factors[factors != layer.drops[i]]
+                    if(length(factors) == 0) {factors <- NULL}
+                }
+                if (is.null(dummy.vars) == F) {
+                    dummy.vars <- dummy.vars[dummy.vars != layer.drops[i]]
+                    if(length(dummy.vars) == 0) {dummy.vars <- NULL}
+                }
+            }
+        }
+    }
+    if (is.null(factors) == F) {
+        vars <- names(x)
+        factors <- as.character(factors)
+        nf <- length(factors)
+        for (i in 1:nf) {
+            if (any(vars==factors[i])==FALSE) {
+                cat(paste("\n", "WARNING: categorical variable '", factors[i], "' not among grid layers", "\n", sep = ""))
+            }
+        }
+    }
+    if (is.null(dummy.vars) == F) {
+        vars <- names(x)
+        dummy.vars <- as.character(dummy.vars)
+        nf <- length(dummy.vars)
+        for (i in 1:nf) {
+            if (any(vars==dummy.vars[i])==FALSE) {
+                cat(paste("\n", "WARNING: dummy variable '", dummy.vars[i], "' not among grid layers", "\n", sep = ""))
+            }
+        }
+    }
+    if (is.null(input.weights)==F) {
+# use the last column in case output from the ensemble.test.splits function is used
+        if (length(dim(input.weights)) == 2) {input.weights <- input.weights[,"MEAN"]}
+        MAXENT <- max(c(input.weights["MAXENT"], -1), na.rm=T)
+        GBM <- max(c(input.weights["GBM"], -1), na.rm=T)
+        RF <- max(c(input.weights["RF"], -1), na.rm=T)
+        GLM <- max(c(input.weights["GLM"], -1), na.rm=T)
+        GLMSTEP <- max(c(input.weights["GLMSTEP"], -1), na.rm=T)
+        GAM <- max(c(input.weights["GAM"], -1), na.rm=T)
+        GAMSTEP <- max(c(input.weights["GAMSTEP"], -1), na.rm=T)
+        MGCV <- max(c(input.weights["MGCV"], -1), na.rm=T)
+        MGCVFIX <- max(c(input.weights["MGCVFIX"], -1), na.rm=T)
+        EARTH <- max(c(input.weights["EARTH"], -1), na.rm=T)
+        RPART <- max(c(input.weights["RPART"], -1), na.rm=T)
+        NNET <- max(c(input.weights["NNET"], -1), na.rm=T)
+        FDA <- max(c(input.weights["FDA"], -1), na.rm=T)
+        SVM <- max(c(input.weights["SVM"], -1), na.rm=T)
+        SVME <- max(c(input.weights["SVME"], -1), na.rm=T)
+        BIOCLIM <- max(c(input.weights["BIOCLIM"], -1), na.rm=T)
+        DOMAIN <- max(c(input.weights["DOMAIN"], -1), na.rm=T)
+        MAHAL<- max(c(input.weights["MAHAL"], -1), na.rm=T)
+        GEODIST <- max(c(input.weights["GEODIST"], -1), na.rm=T)
+    }
     if (MAXENT > 0) {
 	jar <- paste(system.file(package="dismo"), "/java/maxent.jar", sep='')
 	if (!file.exists(jar)) {stop('maxent program is missing: ', jar, '\nPlease download it here: http://www.cs.princeton.edu/~schapire/maxent/')}
     }
     if (formulae.defaults == T) {
-        formulae <- ensemble.formulae(x, factors=factors)
+        formulae <- ensemble.formulae(x, factors=factors, dummy.vars=dummy.vars)
     }
     if (GBM > 0) {
         if (! require(gbm)) {stop("Please install the gbm package")}
@@ -33,7 +108,12 @@
         if (is.null(GBM.formula) == T) {stop("Please provide the GBM.formula (hint: use ensemble.formulae function)")}
         environment(GBM.formula) <- .BiodiversityR
     }
-    if (GBMSTEP > 0) {if (! require(gbm)) {stop("Please install the gbm package")}}
+    if (GBMSTEP > 0) {
+        if (! require(gbm)) {stop("Please install the gbm package")}
+        if (is.null(GBM.formula) == T && formulae.defaults == T) {GBM.formula <- formulae$GBM.formula}
+        if (is.null(GBM.formula) == T) {stop("Please provide the GBM.formula (hint: use ensemble.formulae function)")}
+        environment(GBM.formula) <- .BiodiversityR
+    }
     if (RF > 0) {
         if (! require(randomForest)) {stop("Please install the randomForest package")}
         if (is.null(RF.formula) == T && formulae.defaults == T) {RF.formula <- formulae$RF.formula}
@@ -81,6 +161,14 @@
         assign("GAM.family", GAM.family, envir=.BiodiversityR)
         detach(package:mgcv)   
     }
+    if (MGCVFIX > 0) {
+        if (! require(mgcv)) {stop("Please install the mgcv package")}
+        if (is.null(MGCVFIX.formula) == T && formulae.defaults == T) {MGCVFIX.formula <- formulae$MGCVFIX.formula}
+        if (is.null(MGCVFIX.formula) == T) {stop("Please provide the MGCVFIX.formula (hint: use ensemble.formulae function)")}
+        environment(MGCVFIX.formula) <- .BiodiversityR
+        assign("GAM.family", GAM.family, envir=.BiodiversityR)
+        detach(package:mgcv)   
+    }
     if (EARTH > 0) {
         if (! require(earth)) {stop("Please install the earth package")}
         if (is.null(EARTH.formula) == T && formulae.defaults == T) {EARTH.formula <- formulae$EARTH.formula}
@@ -111,15 +199,26 @@
         if (is.null(SVM.formula) == T) {stop("Please provide the SVM.formula (hint: use ensemble.formulae function)")}
         environment(SVM.formula) <- .BiodiversityR
     }
+    if (SVME > 0) {
+        if (! require(e1071)) {stop("Please install the e1071 package")}
+        if (is.null(SVME.formula) == T && formulae.defaults == T) {SVME.formula <- formulae$SVME.formula}
+        if (is.null(SVME.formula) == T) {stop("Please provide the SVME.formula (hint: use ensemble.formulae function)")}
+        environment(SVME.formula) <- .BiodiversityR
+#         get the probabilities from svm
+            predict.svme <- function(model, newdata) {
+                p <- predict(model, newdata, probability=T)
+                return(attr(p, "probabilities")[,1])
+             }
+    }
     if (is.null(a)==T) {a <- randomPoints(x, n=an, ext=ext)}
     groupp <- kfold(p, k=k)
     groupa <- kfold(a, k=k)
-    output <- array(NA, dim=c(17, k+1))
-    rownames(output) <- c("MAXENT","GBM", "GBMSTEP","RF","GLM", "GLMSTEP", "GAM", "GAMSTEP",
-        "MGCV","EARTH","RPART","NNET","FDA","SVM","BIOCLIM","DOMAIN","MAHAL")
+    output <- array(NA, dim=c(20, k+1))
+    rownames(output) <- c("MAXENT", "GBM", "GBMSTEP", "RF", "GLM", "GLMSTEP", "GAM", "GAMSTEP", "MGCV", "MGCVFIX",
+        "EARTH", "RPART", "NNET", "FDA", "SVM", "SVME", "BIOCLIM", "DOMAIN", "MAHAL", "GEODIST")
     colnames(output) <- c(1:k,"MEAN")
     for (i in 1:k){
-        cat(paste("\n", "EVALUATION RUN: ", i, "\n", "\n", sep = ""))
+        cat(paste("\n", "EVALUATION RUN: ", i, "\n\n", sep = ""))
         pc <- p[groupp != i,]
         pt <- p[groupp == i,]
         ac <- a[groupa != i,]
@@ -129,24 +228,29 @@
         assign("ac", ac, envir=.BiodiversityR)
         assign("at", at, envir=.BiodiversityR)
         tests <- ensemble.test(x=x, p=pc, a=ac, pt=pt, at=at, 
-            PLOTS=PLOTS, evaluations.keep=T,
+            VIF=VIF,
+            PLOTS=PLOTS, evaluations.keep=T, models.keep=F,
             MAXENT=MAXENT, GBM=GBM, GBMSTEP=GBMSTEP, RF=RF, GLM=GLM, GLMSTEP=GLMSTEP, 
-            GAM=GAM, GAMSTEP=GAMSTEP, MGCV=MGCV, EARTH=EARTH, RPART=RPART, 
-            NNET=NNET, FDA=FDA, BIOCLIM=BIOCLIM, DOMAIN=DOMAIN, MAHAL=MAHAL,   
-            Yweights=Yweights, factors=factors,
+            GAM=GAM, GAMSTEP=GAMSTEP, MGCV=MGCV, MGCVFIX=MGCVFIX, EARTH=EARTH, RPART=RPART, 
+            NNET=NNET, FDA=FDA, SVM=SVM, SVME=SVME, BIOCLIM=BIOCLIM, DOMAIN=DOMAIN, MAHAL=MAHAL,
+            GEODIST=GEODIST, GEODIST.file.name=GEODIST.file.name, RASTER.format=RASTER.format,  
+            Yweights=Yweights, factors=factors, dummy.vars=dummy.vars,
+            maxit=maxit,
             GBM.formula=GBM.formula, GBM.n.trees=GBM.n.trees,
             GBMSTEP.gbm.x=GBMSTEP.gbm.x, GBMSTEP.tree.complexity=GBMSTEP.tree.complexity, 
             GBMSTEP.learning.rate=GBMSTEP.learning.rate, GBMSTEP.bag.fraction=GBMSTEP.bag.fraction,
+            GBMSTEP.step.size=GBMSTEP.step.size,
             RF.formula=RF.formula, RF.ntree=RF.ntree, RF.mtry=RF.mtry, 
             GLM.formula=GLM.formula, GLM.family=GLM.family, 
             GLMSTEP.k=GLMSTEP.k, GLMSTEP.steps=GLMSTEP.steps, STEP.formula=STEP.formula, GLMSTEP.scope=GLMSTEP.scope, 
             GAM.formula=GAM.formula, GAM.family=GAM.family, 
             GAMSTEP.steps=GAMSTEP.steps, GAMSTEP.scope=GAMSTEP.scope,
-            MGCV.formula=MGCV.formula,
+            MGCV.formula=MGCV.formula, MGCV.select=MGCV.select,
+            MGCVFIX.formula=MGCVFIX.formula, 
             EARTH.formula=EARTH.formula, EARTH.glm=EARTH.glm,
             RPART.formula=RPART.formula, RPART.xval=RPART.xval, 
             NNET.formula=NNET.formula, NNET.size=NNET.size, NNET.decay=NNET.decay,
-            FDA.formula=FDA.formula, SVM.formula=SVM.formula)
+            FDA.formula=FDA.formula, SVM.formula=SVM.formula, SVME.formula=SVME.formula)
         if(is.null(tests$MAXENT.T)==F) {output["MAXENT",i] <- tests$MAXENT.T@auc}
         if(is.null(tests$GBM.T)==F) {output["GBM",i] <- tests$GBM.T@auc} 
         if(is.null(tests$GBMSTEP.T)==F) {output["GBMSTEP",i] <- tests$GBMSTEP.T@auc} 
@@ -156,31 +260,35 @@
         if(is.null(tests$GAM.T)==F) {output["GAM",i] <- tests$GAM.T@auc} 
         if(is.null(tests$GAMS.T)==F) {output["GAMSTEP",i] <- tests$GAMS.T@auc}
         if(is.null(tests$MGCV.T)==F) {output["MGCV",i] <- tests$MGCV.T@auc} 
+        if(is.null(tests$MGCVF.T)==F) {output["MGCVFIX",i] <- tests$MGCVF.T@auc} 
         if(is.null(tests$EARTH.T)==F) {output["EARTH",i] <- tests$EARTH.T@auc} 
         if(is.null(tests$RPART.T)==F) {output["RPART",i] <- tests$RPART.T@auc}
         if(is.null(tests$NNET.T)==F) {output["NNET",i] <- tests$NNET.T@auc} 
         if(is.null(tests$FDA.T)==F) {output["FDA",i] <- tests$FDA.T@auc}
         if(is.null(tests$SVM.T)==F) {output["SVM",i] <- tests$SVM.T@auc}
+        if(is.null(tests$SVME.T)==F) {output["SVME",i] <- tests$SVME.T@auc}
         if(is.null(tests$BIOCLIM.T)==F) {output["BIOCLIM",i] <- tests$BIOCLIM.T@auc}
         if(is.null(tests$DOMAIN.T)==F) {output["DOMAIN",i] <- tests$DOMAIN.T@auc}
         if(is.null(tests$MAHAL.T)==F) {output["MAHAL",i] <- tests$MAHAL.T@auc}
+        if(is.null(tests$GEODIST.T)==F) {output["GEODIST",i] <- tests$GEODIST.T@auc}
     }
     output[,k+1] <- rowMeans(output[,1:k], na.rm=T)
-    outp <- round(output, digits=4)
-    outp[is.na(outp[,"MEAN"]),] <- 0  
-    if (SCRIPT==TRUE) {
-        cat(paste("\n", "Suggested parameters for ensemble.grd function", "\n", sep = ""))
-        cat(paste("\n", "(these are based on average AUC)", "\n", "\n", sep=""))
-        cat(paste("\n", "MAXENT=", outp[1, "MEAN"], ", GBM=", outp[2, "MEAN"], ", GBMSTEP=", outp[3, "MEAN"], ", RF=", outp[4, "MEAN"], 
-            ", GLM=", outp[5, "MEAN"], ", GLMSTEP=", outp[6, "MEAN"],  
-            ", GAM=", outp[7, "MEAN"], ", GAMSTEP=", outp[8, "MEAN"], ",", "\n",
-            "MGCV=", outp[9, "MEAN"], ", EARTH=", outp[10, "MEAN"], ", RPART=", outp[11, "MEAN"],
-            ", NNET=", outp[12, "MEAN"], ", FDA=", outp[13, "MEAN"], ", SVM=", outp[14, "MEAN"], ", BIOCLIM=", outp[15, "MEAN"],
-            ", DOMAIN=", outp[16, "MEAN"], ", MAHAL=", outp[17, "MEAN"], "\n", "\n", "\n", sep=""))
-    }
-    output <- output[order(output[,"MEAN"], decreasing=T),]
+    output[is.na(output[,"MEAN"]),] <- 0  
     output <- 100*output
     output <- round(output, digits=digits)
+    if (SCRIPT==TRUE) {
+        weights <- output[,"MEAN"]
+# no GEODIST
+        weights <- weights[-20]
+    }
+    output <- output[order(output[,"MEAN"], decreasing=T),]
+    cat(paste("\n", "Results (as percentage) of ensemble.test.splits sorted by average AUC ",  "\n\n", sep = ""))
+    print(output)
+    if (SCRIPT==TRUE) {
+        cat(paste("\n", "Suggested parameters for ensemble.grd function", "\n", sep = ""))
+        cat(paste("(these are based on average AUC)", "\n\n", sep=""))
+        print(weights)
+    }
     remove(pc, envir=.BiodiversityR)
     remove(pt, envir=.BiodiversityR)
     remove(ac, envir=.BiodiversityR)
